import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.midi.*;
import java.util.*;

/**
 * Graphical representation of a MIDI track.
 * 
 * @author  Andersson , Andreen, Lanneskog, Pehrson
 * @version 1
 */

public class MooTrackView extends JPanel {

	private Track track;
	private Rectangle box;

	private JPopupMenu popup;
	private JMenuItem popupAdd;
	private ArrayList rects;
	protected static int viewLength = 0;
	protected static int extraHeight = 0;
	public static final int NOTE_HEIGHT = 10, NOTE_WIDTH = 40, VIEW_WIDTH = 200;

	public MooTrackView (Track track) {
		super(true);
		this.track = track;

		// Configures panel
		setBackground(Color.white);
		setBorder(BorderFactory.createLineBorder(Color.black));
		setLayout(null);
		setPreferredSize(new Dimension(VIEW_WIDTH, 140 * NOTE_HEIGHT));

		// Creates temporary variables
		MidiEvent note;
		MooNoteElement elem;
		rects = new ArrayList(track.size() / 2);
		extraHeight = Toolkit.getDefaultToolkit().getScreenSize().height - 150;

		// Places note elements
		for (int i = 0; i < track.size(); i++) {
			note = track.get(i);
			if (note instanceof MooNote) {
				// Adds the note element to the note area and moves it to the appropriate place.
				MooNote mn = (MooNote)note;
				elem = new MooNoteElement(this, mn);
				add(elem);
				layoutElement(elem, false);
			}
			setPreferredSize(new Dimension(VIEW_WIDTH, viewLength + extraHeight));

		}
		validate();

		// Creates pop-up menu.
		popup = new JPopupMenu();
		PopupListener pList = new PopupListener();
		popupAdd = new JMenuItem("Add note...");
		popupAdd.addActionListener(pList);
		popup.add(popupAdd);

		// Adds listeners for popup menu and keyboard synthesizer.
		addMouseListener(new MAdapter());
		addKeyListener(new MooKeyboard());
	}

	public void layoutElementMooNoteElement elem, boolean old){
		// If the element is currently in the view, removes its coordinates from the list.
		Rectangle r = new Rectangle();
		if (old){
			r = elem.getBounds(r);
			for (Iterator i = rects.iterator(); i.hasNext();){
				Object ob = i.next();
				if (r.equals(ob)){
					rects.remove(ob);
					break;
				}
			}
		}

		// Creates temporary variables.
		int ticksPerSixteenth = Moosique.getSequence().getResolution() / 4;
		MooNote mn = elem.getNote();
		Insets insets = getInsets();
		int x, y, height;

		// Calculates coordinates.
		x = insets.left;
		y = insets.top + (int)(mn.getTick() / ticksPerSixteenth) * NOTE_HEIGHT;
		height = (mn.getDuration() / ticksPerSixteenth) * NOTE_HEIGHT;
		if (height == 0) height = NOTE_HEIGHT;
		r = new Rectangle(x, y, NOTE_WIDTH, height);

		// Places the element in the appropriate place.
		while(isOccupied(r)) r.translate(NOTE_WIDTH, 0);
		elem.setBounds(r);
		rects.add(r);
		if (viewLength < (y + height)){
			viewLength = y + height;
			if(old)setPreferredSize(new Dimension(VIEW_WIDTH, viewLength + extraHeight));
		}

	}

	public Track getTrack() {
		return track;
	}


	/** 
	 * Updates the track view.
	 */
	public void update(long tickPosition) {
		repaint();
	}

	private boolean isOccupied(Rectangle r) {
		Iterator it = rects.iterator();
		while (it.hasNext()) {
			if(r.intersects((Rectangle)it.next())) return true;
		}
		return false;
	}
	
	public void addNote(MooNote mn) {
		mn.addTo(track);
		MooNoteElement elem = new MooNoteElement(this, mn);
		add(elem);
		layoutElement(elem, false);
		setPreferredSize(new Dimension(VIEW_WIDTH, viewLength + extraHeight));
	}

	public void removeNote(MooNoteElement elem, MooNote mn) {
		mn.removeFrom(track);
		remove(elem);
		elem.getNote().removeFrom(track);
		validate();
		repaint();
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D)g;
		for (int c = 0; c < viewLength || c < getHeight(); c += NOTE_HEIGHT) {
			for (int r = 0; r < (10 * NOTE_WIDTH); r += NOTE_WIDTH) {
				box = new Rectangle(r, c, NOTE_WIDTH, NOTE_HEIGHT);
				g2.setColor(Color.gray);
				g2.draw(box);
			}
		}
	}

	class MAdapter extends MouseAdapter {
		public void mousePressed(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}

		public void mouseEntered(MouseEvent e) {
			// Moosique.setActiveChannel(track.getChannel());
			grabFocus();
		}
	}

	class PopupListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if  (source == popupAdd) {
				addNote(new MooNote());
				// int channel, int pitch, int velocity, long timestamp, int duration
		}
	}
}